/// <reference types="node" />
import { EventEmitter } from 'events';
import firebase from 'firebase';
import SimplePeer from 'simple-peer';
/**
 * The firepeer instance, just a [simple-peer](https://github.com/feross/simple-peer#api)
 * instance in connected state.
 *
 * @noInheritDoc
 */
export interface FirePeerInstance extends SimplePeer.Instance {
    /**
     * The uid of the initiator of the connection
     */
    initiatorUid: string | null;
    /**
     * The peer id of the initiator of the connection
     */
    initiatorId: string | null;
    /**
     * The uid of the receiver of the connection
     */
    receiverUid: string | null;
    /**
     * The peer id of the receiver of the connection
     */
    receiverId: string | null;
}
/**
 * Represents p2p signalling data. `error` is specific to firepeer and enables passing error information.
 */
export interface Signal extends SimplePeer.SignalData {
    type: 'offer' | 'answer' | 'error';
    /**
     * The uid of the peer where the signal came from
     */
    uid: string;
    /**
     * The id of the peer where the signal came from
     */
    id: string;
}
/**
 * Intercepts signals to/from firepeer instances.
 * For the signalling to proceed, firepeer expects the function to return a
 * Promise resolving to a signal or a plain signal.
 * Otherwise, return null or a promise rejection.
 */
export declare type SignalInterceptor = (signal: Signal) => Promise<Signal> | Signal | null;
export interface FirePeerOptions {
    /**
     * The peer id to be used. Should be unique. If not specified, then a generated one will be used.
     */
    id?: string;
    /**
     * Simple-peer [constructor options](https://github.com/feross/simple-peer#api).
     */
    spOpts?: SimplePeer.Options;
    /**
     * [[SignalInterceptor]] called when an offer is received from the initiator.
     */
    onOffer?: SignalInterceptor;
    /**
     * [[SignalInterceptor]] called when an answer is received from the receiver.
     */
    onAnswer?: SignalInterceptor;
    /**
     * [[SignalInterceptor]] called when an offer is about to be sent by the initiator.
     */
    sendOffer?: SignalInterceptor;
    /**
     * [[SignalInterceptor]] called when an answer is about to be sent by the receiver.
     */
    sendAnswer?: SignalInterceptor;
}
export declare interface FirePeer {
    /**
     * Triggered when a new connection is established either initiated by you through [[FirePeer.connect]]
     * or initiated by another peer.
     */
    on(event: 'connection', listener: (peer: FirePeerInstance) => void): this;
    /**
     * Triggered when a connection failed either through [[FirePeer.connect]]
     * or initiated by another peer.
     */
    on(event: 'connection_failed', listener: (error: Error) => void): this;
    /**
     * Triggered when firebase auth state has changed. Firepeer only works if firebase is in authenticated state
     * ([[FirePeer.uid]] is not null).
     */
    on(event: 'loggedin' | 'loggedout', listener: () => void): this;
}
/**
 * Secure p2p signalling and authentication for [simple-peer](https://github.com/feross/simple-peer)
 * using [firebase realtime database](https://firebase.google.com/docs/database/).
 *
 * @noInheritDoc
 */
export declare class FirePeer extends EventEmitter {
    /**
     * A unique string identifying this peer from other peers. Used as the `id` parameter in [[FirePeer.connect]].
     */
    id: string;
    /**
     * A unique string assigned by firebase when in authenticated state. This identifies the current user within firebase.
     * Used as the `uid` parameter in [[FirePeer.connect]].
     */
    uid?: string | null;
    private app;
    private ref?;
    private spOpts?;
    private onOffer;
    private onAnswer;
    private sendOffer;
    private sendAnswer;
    /**
     *
     * @param fbaseOrApp Configured firebase instance or a specific firebase app if you have configured multiple apps.
     */
    constructor(fbaseOrApp: firebase.app.App | any, options?: FirePeerOptions);
    /**
     * Connect to a peer identified by a user id and peer id. Returns a promise that resolves to a [[FirePeerInstance]].
     */
    connect(uid: string, id: string): Promise<FirePeerInstance>;
    private listen;
    private unlisten;
    private createPeer;
}
